// Definition of the TLS class

#ifndef TLS_class
#define TLS_class

#include "tls/aes/aes.h"

//arch linux
#include "tls/arch/linux/ClientSocket.h"
#include "tls/arch/linux/SocketException.h"

//ISA independent
const int _test_endian = 1;
#define is_bigendian() ( (*(char*)&_test_endian) == 0 )

//arch independent
#include <stdint.h>

typedef unsigned char u8;
typedef unsigned short u16; //unsigned short uint6_t 

typedef unsigned int u32; //unsigned int uint32_t


/*unsigned char* makeS24( unsigned int i, unsigned char* s24 )
{
    s24[2] = (unsigned char)(i >> 16) ;
    s24[1] = (unsigned char)((i >> 8) & 0xff);
    s24[0] = (unsigned char)(i & 0xff);
    return s24 ;
}*/





/*-- Class RecordLayer --- */
/*

--- RFC ---

struct {
           uint8 major, minor;
       } ProtocolVersion;

       enum {
           change_cipher_spec(20), alert(21), handshake(22),
           application_data(23)
       } ContentType;

       struct {
           ContentType type;
           ProtocolVersion version;
           uint16 length;
           opaque fragment[TLSPlaintext.length];
       } TLSPlaintext;
*/
class RecordLayer{

public:

    // type
    /*The higher-level protocol used to process the enclosed fragment.
      Samples: //change_cipher_spec(20), alert(21), handshake(22), application_data(23), */	
    u8 _content_type; 
    
    //version
    /*The version of the protocol being employed.  This document
      describes TLS Version 1.1, which uses the version { 3, 2 }.  The
      version value 3.2 is historical: TLS version 1.1 is a minor
      modification to the TLS 1.0 protocol, which was itself a minor
      modification to the SSL 3.0 protocol, which bears the version
      value 3.0.  (See Appendix A.1.)*/  
    u8 _version_major;// first digit, e.g., TLS 1.2: 3.0 => (3) 
    u8 _version_minor;// seconf digit, e.g, TLS 1.2: 3.0 => (0) *See Table
   
    //length
    /*The length (in bytes) of the following TLSPlaintext.fragment.  The
    length should not exceed 2^14.*/
    u16 _length;      // max lenght in theory (64Kb)

    int montarMsg(char** dt, int* size);

    void recuperarMsg(const char* data);
};




/*-- Class ApplicationData --- */

class ApplicationData : public RecordLayer {

public:
    char* msg;
    int montarMsg(char** dt, int* size);
    ApplicationData* recuperarMsg(const char* msg, u16 len);
};





/*-- Class HandshakeProtocol --- */
/*
 The TLS Handshake Protocol is one of the defined higher-level clients
   of the TLS Record Protocol.  This protocol is used to negotiate the
   secure attributes of a session.  Handshake messages are supplied to
   the TLS Record Layer, where they are encapsulated within one or more
   TLSPlaintext structures, which are processed and transmitted as
   specified by the current active session state.

enum {
          hello_request(0), client_hello(1), server_hello(2),
          certificate(11), server_key_exchange (12),
          certificate_request(13), server_hello_done(14),
          certificate_verify(15), client_key_exchange(16),
          finished(20)
      } HandshakeType;

      struct {
          HandshakeType msg_type;    /* handshake type * /
          uint24 length;             /* bytes in message * /
          select (HandshakeType) {
              case hello_request:       HelloRequest;
              case client_hello:        ClientHello;
              case server_hello:        ServerHello;
              case certificate:         Certificate;
              case server_key_exchange: ServerKeyExchange;
              case certificate_request: CertificateRequest;
              case server_hello_done:   ServerHelloDone;
              case certificate_verify:  CertificateVerify;
              case client_key_exchange: ClientKeyExchange;
              case finished:            Finished;
          } body;
      } Handshake;
*/
class HandshakeProtocol : public RecordLayer {

public:

    u8 messageType;    	   // sample:  hello_request(0), client_hello(1), server_hello(2), etc... 	
    u8 handshakeLength[3]; // 3 bytes 

    int montarMsg(char** dt, int* size);
    void recuperarMsg(const char* data);
};



/*-- Class ClientHello --- */
/*struct {
          ProtocolVersion client_version;
          Random random;
          SessionID session_id;
          CipherSuite cipher_suites<2..2^16-1>;
          CompressionMethod compression_methods<1..2^8-1>;
      } ClientHello;*/
class ClientHello : public HandshakeProtocol {

public:

    /*gmt_unix_time
      The current time and date in standard UNIX 32-bit
      format (seconds since the midnight starting Jan 1, 1970, GMT,
      ignoring leap seconds) according to the sender's internal clock.
      Clocks are not required to be set correctly by the basic TLS
      Protocol; higher-level or application protocols may define
      additional requirements.*/
    u32 gmt_unix_time; //4 bytes
    
    //random_bytes
    //28 bytes generated by a secure random number generator.
    u32 random_bytes[7];
    
    u8 cipher_suite0; 
    u8 cipher_suite1; 

    //session_id
    /*  The ID of a session the client wishes to use for this connection.
      This field should be empty if no session_id is available or if the
      client wishes to generate new security parameters.*/
    u8 session_id[32]; 

    int montarMsg(char** dt, int* size);
};

class ServerHello : public HandshakeProtocol {

public:
    u8 gmt_unix_time[4]; //4 bytes
    u8 random_bytes[28]; //28 bytes	
    u8 session_id_lenght;
    u8 session_id[32]; 
    u8 cipher_suite[2];	
    u8 compression_method;

    void recuperarMsg(const char* data);
};


class Certificate : public HandshakeProtocol {

public:
    u8 content_lenght[3];
    u8 session_id[5049]; 

    void recuperarMsg(const char* data);
};





/*-- Class ChangeCipherSpec --- */

class ChangeCipherSpec : public RecordLayer {
public:
    u8 change_cipher_spec;
    int montarMsg(char** dt, int* size);
    ChangeCipherSpec* recuperarMsg(const char* msg, u16 len);
};


class ClientKeyExchange : public HandshakeProtocol {
 public:
    u8 psk_identity;
    int montarMsg(char** dt, int* size);
    ClientKeyExchange* recuperarMsg(const char* msg, u16 len);
};

class Finished : public HandshakeProtocol {

public:
    char* verify_data;
    int verify_data_size;
    int montarMsg(char** dt, int* size);
    Finished* recuperarMsg(const char* msg, u16 len);
};


class TLS
{
 public:

  TLS (char* h);
  ~TLS();

  //mtodos arch linux
  void tcp_conectar();
  char* tcp_send_receive(const char* in, int s);
  void tcp_send(const char* in, int s);
  char* tcp_receive(int* size);

  char* send_receive(char* message, int s);
  void teste_aes();
  
  // estados do cliente
  enum {
		START, 
		HELLO_SENT,
		KEY_EXCHANGE,
		SEND_FINISHED,
		WAIT_FINISHED,
		READY,
		SEND_DATA,
		RECV_DATA,
		CLOSE
    };
	
    // funes dos estados
    
	void _START();
	void _HELLO_SENT();
    	void _KEY_EXCHANGE();
	void _SEND_FINISHED();
    	void _WAIT_FINISHED();
    	void _READY();
	void _SEND_DATA();
    	void _RECV_DATA();
    	void _CLOSE();
  
private:
    
    u8* master_secret;
    //master secret
    //48-byte secret shared between the client and server.
    int master_secretSize;


    u8* psk;
    int pskSize;
    u8* pre_master_secret;
    int pmsSize;
    
    //client random
    //A 32-byte value provided by the client.
    u32 randomClient[8];
    
    //server random
    //A 32-byte value provided by the server.
    u32 randomServer[8];

    char* dados;
    u16 tamanhoDados;

    char* recebido;
    u16 tamanhoRecebido;

    char * finished;
    int size;

   char* host;
   u8 _state;
   u32 _gmt_unix_time;


  u8 state(){return this->_state;}
  void setState(u8 n){this->_state = n;}

  //arch linux
  ClientSocket* client_socket;
};


#endif
